#ifndef XML_H_
#define XML_H_

#include <glib-object.h>

typedef struct _StateInfo	StateInfo;
typedef struct _PropInfo	PropInfo;
typedef struct _PageInfo	PageInfo;
typedef struct _WidgetInfo	WidgetInfo;
typedef struct _SignalInfo	SignalInfo;
typedef struct _GroupWidgetInfo	GroupWidgetInfo;
typedef struct _StyleInfo	StyleInfo;
typedef struct _LinkAttributeInfo	LinkAttributeInfo;
typedef struct _TemplateInfo	TemplateInfo;
typedef struct _TemplateInUse	TemplateInUse;
typedef struct _ResultInfo	ResultInfo;

typedef enum {
	START_STATE,
	FINISH_STATE,
	WPS_STATE,
	PAGE_STATE,
	WIDGET_STATE,
	GROUP_STATE,
	GROUP_WIDGET_STATE,
	WIDGET_PROPERTY_STATE,
	GROUP_PROPERTY_STATE,
	GROUP_WIDGET_PROPERTY_STATE,
	PAGE_PROPERTY_STATE,
	GROUP_WIDGET_STYLE_STATE,
	SIGNAL_STATE,
	WIDGET_STYLE_STATE,
	WIDGET_LINK_ATTRIBUTE_STATE,
	TEMPL_STATE,
	TEMPL_WIDGET_STATE,
	TEMPL_WIDGET_PROPERTY_STATE,
	TEMPL_WIDGET_SIGNAL_STATE,
	TEMPL_WIDGET_STYLE_STATE,
	TEMPL_WIDGET_LINK_ATTRIBUTE_STATE,
	PAGE_TEMPL,
	GROUP_SIGNAL_STATE,
	GROUP_LINK_ATTRIBUTE_STATE,
	UNKNOWN_STATE
} ParserState; 


struct _PropInfo {
	GString *name;
	GString *value;
};

struct _TemplateInfo {
	gchar	*name;
	gchar	*desc;
	GList	*widget_info;
	GList	*group_widget_info;
};

struct _TemplateInUse {
	gchar	*name;
	gint	xalign;
	gint	yalign;
};

struct _SignalInfo {
	GString *handler;
};

struct _StyleInfo {
	gint font_size;
	gchar *field_color;
	gchar *field_font_color;
};

struct _LinkAttributeInfo {
	GString *name;
	GString *tag;
	GString *type;
	GString *direction;
};

struct _WidgetInfo {
	GString	*name;
	GString	*type_name;
	GArray *prop_info_array;
	GList *signal_info;
	StyleInfo *style_info;
	GList *link_attributes_info;
	gint prop_len;
	gint64 xalign;
	gint64 yalign;
};

struct _GroupWidgetInfo {
	GString	*name;
	GList *widget_info;
	GList *signal_info;
	GList *link_attributes_info;
	GArray *prop_info_array;
	gint prop_len;
};

struct _PageInfo {
	GString	*name;
	GString	*description;
	GArray *page_prop_info;
	gint	prop_len;
	GList	*widget_info;
	GList	*group_widget_info;
	GList	*templ_use;
};

struct _StateInfo {
	ParserState	state;
	ParserState	prev_state;
	gint 		unknown_depth;
	PageInfo 	*cur_page;
	WidgetInfo 	*cur_widget;
	SignalInfo	*cur_signal;
	GList		*page_info;
	GList		*template_info;
};

struct _ResultInfo {
	GList *page_info;
	GList *template_info;
};

ResultInfo		*parse_xml_file(gchar *file);

#endif /*XML_H_*/
